﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mfe
{
    public partial class FontTextPanel : UserControl
    {
        public FontTextPanel()
        {
            InitializeComponent();
            SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.OptimizedDoubleBuffer | ControlStyles.ResizeRedraw | ControlStyles.UserPaint, true);
        }

        protected Font font;
        public Font CurrentFont
        {
            get
            {
                return font;
            }
            set
            {
                font = value;
                Invalidate();
            }
        }

        protected int scaleFactor = 2;
        public int ScaleFactor
        {
            get
            {
                return scaleFactor;
            }
            set
            {
                scaleFactor = value;
                Invalidate();
            }
        }

        protected int screenWidth = 96;
        public int ScreenWidth
        {
            get
            {
                return screenWidth;
            }
            set
            {
                screenWidth = value;
                Invalidate();
            }
        }

        protected int screenHeight = 64;
        public int ScreenHeight
        {
            get
            {
                return screenHeight;
            }
            set
            {
                screenHeight = value;
                Invalidate();
            }
        }
        protected string[] lines = new string[] {"The quick brown fox jumps over the lazy dog."};
        public string[] Lines
        {
            get
            {
                return lines;
            }
            set
            {
                if (value != null && value.Length > 0)
                    lines = value;
            }
        }

        protected SolidBrush foreBrush = new SolidBrush(Color.Black);
        protected override void OnForeColorChanged(EventArgs e)
        {
            base.OnForeColorChanged(e);
            foreBrush.Color = this.ForeColor;
        }

        protected SolidBrush screenBackBrush = new SolidBrush(Color.White);

        SolidBrush backBrush = new SolidBrush(Color.White);
        public Color VirtualBackColor
        {
            get
            {
                return backBrush.Color;
            }
            set
            {
                backBrush.Color = value;
            }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if (font == null)
                return;
            int x = 0;
            int y = 0;
            
            float scalex = (float)this.scaleFactor * font.AspectRatioWidth;
            float scaley = (float)this.scaleFactor * font.AspectRatioHeight;
            e.Graphics.FillRectangle(backBrush, x, y, (screenWidth) * scalex, scaley * (screenHeight));
            bool ignoreNextNewline = false;
            bool newLine = false;
            char currentChar;
            for (int j = 0; j < this.Lines.Length; j++)
            {
                for (int i = 0; i < this.Lines[j].Length; i++)
                {
                    currentChar = this.Lines[j][i];
                    if ((int)currentChar == 13)
                    {
                        ignoreNextNewline = true;
                        x = 0;
                        y = y + font.Height;
                        if (y >= screenHeight)
                            return;
                        continue;
                    }
                    else if ((int)currentChar == 10 && !ignoreNextNewline)
                    {
                        x = 0;
                        y = y + font.Height;
                        if (y >= screenHeight)
                            return;
                        continue;
                    }
                    if (newLine)
                    {
                        newLine = false;
                        x = 0;
                        y = y + font.Height;
                        if (y >= screenHeight)
                            return;
                    }
                    ignoreNextNewline = false;

                    if (font[currentChar] == null)
                        continue;

                    if (font[currentChar].LogicalWidth + x > screenWidth)
                    {
                        x = 0;
                        y = y + font.Height;
                        if (y >= screenHeight)
                            return;
                    }
                    font[currentChar].PaintVirtual(x, y, screenWidth, screenHeight, scalex, scaley, foreBrush, null, e);
                    x += font[currentChar].LogicalWidth;
                }
                newLine = true;
            }
        }
    }
}
