﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Mfe
{
    public partial class MasterWindow : Form
    {
        public Char ClipboardChar = null;
        protected Font currentFont = null;
        public Font CurrentFont
        {
            get
            {
                return currentFont;
            }
            set
            {
                //if (CharEditorWindow != null)
                //    CharEditorWindow.CurrentFont = value;
                currentFont = value;
                CurrentFontOriginalPath = "";
            }
        }
        public string CurrentFontOriginalPath = "";

        public CharEditorWindow CharEditorWindow = null;
        public FontPropertiesEditor FontPropertiesEditor = null;
        public ExportsWindow ExportsWindow = null;
        public PreviewChart PreviewChart = null;
        public HelpWindow HelpWindow = null;

        public MasterWindow()
        {
            InitializeComponent();
            CurrentFont = new Font();
            CharEditorWindow = new CharEditorWindow(this);
            CharEditorWindow.MdiParent = this;
            CharEditorWindow.Show();
        }

        private void loadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (openFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    currentFont = Mfe.Font.ReadFromFile(CurrentFontOriginalPath = openFileDialog.FileName);
                    RefreshStuff();
                }
            }
            catch (Exception x)
            {
                MessageBox.Show("Error loading file: " + x.ToString(), "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        public void RefreshStuff()
        {
            if (CharEditorWindow != null)
                CharEditorWindow.RefreshData();
            if (FontPropertiesEditor != null)
                FontPropertiesEditor.RefreshData();
            if (PreviewChart != null)
                PreviewChart.RefreshData();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                if (currentFont == null)
                    return;
                if (CurrentFontOriginalPath != null && !CurrentFontOriginalPath.Equals(""))
                    currentFont.WriteToFile(CurrentFontOriginalPath);
                else
                    if (saveFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                        currentFont.WriteToFile(CurrentFontOriginalPath = saveFileDialog.FileName);
            }
            catch (Exception x)
            {
                MessageBox.Show("Error saving file: " + x.ToString(), "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentFont == null)
                CurrentFont = new Font();
            else
                if (MessageBox.Show("Any changes after the last save will be lost. Proceed?", "Font May be Lost", MessageBoxButtons.OKCancel) == System.Windows.Forms.DialogResult.OK)
                    CurrentFont = new Font();
                else
                    return;
            RefreshStuff();
            characterEditorToolStripMenuItem_Click(sender, e);
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void chartToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentFont == null)
                return;
            if (PreviewChart == null)
                new PreviewChart(this);
            else
                PreviewChart.BringToFront();
        }

        private void characterEditorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentFont == null)
                return;
            if (CharEditorWindow == null)
            {
                CharEditorWindow = new CharEditorWindow(this);
                CharEditorWindow.MdiParent = this;
                CharEditorWindow.Show();
            }
            else
            {
                CharEditorWindow.BringToFront();
            }
        }

        private void fontPropertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentFont == null)
                return;
            if (FontPropertiesEditor == null)
            {
                FontPropertiesEditor = new FontPropertiesEditor(this);
                FontPropertiesEditor.MdiParent = this;
                FontPropertiesEditor.Show();
            }
            else
                FontPropertiesEditor.BringToFront();
        }

        private void textPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PreviewText x = new PreviewText(this);
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try 
            {
                if (currentFont == null)
                    return;
                if (saveFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    currentFont.WriteToFile(CurrentFontOriginalPath = saveFileDialog.FileName);
            }
            catch (Exception x)
            {
                MessageBox.Show("Error saving file: " + x.ToString(), "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void exportCodeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentFont == null)
                return;
            if (ExportsWindow == null)
            {
                ExportsWindow = new ExportsWindow(this);
            }
            else
                ExportsWindow.BringToFront();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Monochrome Font Editor\n23 August 2013", "About MFE");
        }

        private void readMeFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (HelpWindow == null)
            {
                HelpWindow = new HelpWindow(this);
            }
            else
                HelpWindow.BringToFront();
        }

        private void importToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Importer importer = new Importer();
            if (importer.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                if (importer.ImportedFont != null)
                    CurrentFont = importer.ImportedFont;
        }

        private void importBDFToolStripMenuItem_Click(object sender, EventArgs e)
        {
            BdfImporter importer = new BdfImporter(this);
            if (importer.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                currentFont = importer.ImportedFont;
                RefreshStuff();
            }
            
        }
    }
}
